@@ -30,14 +30,18 @@ from roguewave.wavetheory.lineardispersion import \
     inverse_intrinsic_dispersion_relation, \
     jacobian_wavenumber_to_radial_frequency
 from roguewave.wavewatch3.restart_file_metadata import MetaData
-from typing import Sequence, Union, Tuple, Mapping
+from typing import Sequence, Union, Tuple, Dict
 from roguewave.interpolate.nd_interp import NdInterpolator
 from datetime import datetime
 from functools import cache
 from roguewave.tools.time import to_datetime_utc, to_datetime64
 from roguewave.interpolate.geometry import TrackSet
-from xarray import DataArray, Dataset, concat
+from xarray import Dataset, concat
+from multiprocessing.pool import ThreadPool
+from tqdm import tqdm
+from roguewave.wavespectra import FrequencyDirectionSpectrum
 
+MAXIMUM_NUMBER_OF_WORKERS = 10
 
 class RestartFile(Sequence):
     _start_record = 2
@@ -47,13 +51,15 @@ class RestartFile(Sequence):
                  meta_data:MetaData,
                  resource:Resource,
                  depth:LinearIndexedGridData=None,
-                 return_freq_energy_density=True):
+                 return_freq_energy_density=True,
+                 parallel=True):
 
         self._grid = grid
         self._meta_data = meta_data
         self.resource = resource
         self._dtype = numpy.dtype("float32").newbyteorder(meta_data.byte_order)
         self._convert = return_freq_energy_density
+        self.parallel = parallel
 
         if depth is None:
             _depth = numpy.inf \
@@ -316,7 +322,7 @@ class RestartFile(Sequence):
 
     def interpolate_in_space(self,
                              latitude:Union[numpy.ndarray,float],
-                             longitude:Union[numpy.ndarray,float]) -> numpy.ndarray:
+                             longitude:Union[numpy.ndarray,float]) -> Dataset:
         """
         Extract interpolated spectra at given latitudes and longitudes.
         Input can be either a single latitude and longitude pair, or a
@@ -334,9 +340,6 @@ class RestartFile(Sequence):
         points = {"latitude":numpy.atleast_1d(latitude),
                   "longitude":numpy.atleast_1d(longitude)}
 
-        coordinates = [("latitude",self.latitude),
-                       ("longitude",self.longitude)]
-
         periodic_coordinates = {"longitude":360}
 
         def _get_data(  indices, _dummy ):
@@ -347,7 +350,8 @@ class RestartFile(Sequence):
                                      self.number_of_frequencies,
                                      self.number_of_directions ) )
             mask = index >= 0
-            output[ mask,:,:] = self.__getitem__(index[mask])
+            output[ mask,:,:] = \
+                self.__getitem__(index[mask])['variance_density'].values
             output[~mask,:,:] =numpy.nan
             return output
 
@@ -371,8 +375,8 @@ class RestartFile(Sequence):
             data_period=None,
             data_discont=None
         )
-
-        return interpolator.interpolate(points)
+        return self._create_dataset( interpolator.interpolate(points),
+                                     numpy.arange(len(latitude)) )
 
     def to_wavenumber_action_density(
             self, s:Union[slice, numpy.ndarray, Sequence] ) -> numpy.array:
@@ -392,7 +396,7 @@ class RestartFile(Sequence):
 
     def to_frequency_energy_density(
             self, s:Union[slice, numpy.ndarray, Sequence]
-                                    ) -> numpy.array:
+                                    ) -> numpy.ndarray:
 
         """
         Factor that when multiplied with wavenumber action density spectra at
@@ -508,7 +512,7 @@ class RestartFile(Sequence):
         toggle = not self._convert
         if toggle:
             self.set_return_freq_energy_density()
-        spectra = self[index]
+        spectra = self['variance_density'][index].values
         if toggle:
             self.set_return_k_action_density()
 
@@ -537,12 +541,43 @@ class RestartFile(Sequence):
                self.number_of_spatial_points * \
                self.number_of_spectral_points * self._dtype.itemsize
 
+    def linear_indices(self,indices:Union[slice,numpy.ndarray,Sequence]):
+        return numpy.arange(self.number_of_spatial_points)[indices]
+
+    def _create_dataset(self, variance_density,
+                        linear_indices: Union[slice,numpy.ndarray,Sequence]):
+
+        coords = self.coordinates(linear_indices)
+        return Dataset(
+            data_vars={
+                "variance_density": (
+                    ('point_index','frequency','direction'),variance_density),
+                "longitude": (
+                    ('point_index'),coords[1] ),
+                "latitude": (
+                    ('point_index'),coords[0]),
+                "linear_index": (
+                    ('point_index'),linear_indices),
+                "time": self.time
+            },
+            coords={
+                "point index": numpy.arange(0,len(linear_indices) ),
+                'frequency':self.frequency,
+                'direction':self.direction
+            })
+
+
 
 class RestartFileStack:
-    def __init__(self, restart_files: Sequence[RestartFile]):
+    def __init__(self, restart_files: Sequence[RestartFile],parallel = True):
         self._restart_files = restart_files
         self._grid = restart_files[0]._grid
         self._time = to_datetime_utc([ x.time for x in restart_files ])
+        self.parallel = parallel
+
+        # To make the progress bar work we need to store progress across
+        # calls somewhere.
+        self._progres = {"position":0, "total":None, "leave":True}
 
     @property
     def frequency(self) -> numpy.ndarray:
@@ -636,7 +671,7 @@ class RestartFileStack:
     def __len__(self):
         return len(self._restart_files)
 
-    def __getitem__(self, nargs):
+    def __getitem__(self, nargs) -> FrequencyDirectionSpectrum:
         if len(nargs) == 2:
             time_index, linear_index = nargs
         elif len(nargs) == 3:
@@ -649,24 +684,69 @@ class RestartFileStack:
         if isinstance(time_index,(int,numpy.int32,numpy.int64)):
             time_index = [time_index]
 
-        if isinstance(time_index,slice):
-            time = self.time[time_index]
-            time_index = list(range(*time_index.indices(len(self))))
-            data = [self._restart_files[it][linear_index] for it in time_index]
+        fancy_index = not isinstance(time_index,slice)
+        if fancy_index:
+            _input = list(zip( time_index,linear_index ))
         else:
-            time = [self.time[it] for it in time_index]
-            data = [self._restart_files[it][ilin] for it,ilin
-                    in zip(time_index,linear_index)]
+            time_index = list(range(*time_index.indices(len(self))))
+            _input = [ (it, linear_index) for it in time_index]
+
+        def _worker( arg ):
+            return self._restart_files[arg[0]][arg[1]]
+
+        self._init_progress_bar(len(_input))
+        if self.parallel and len(_input) > 1:
+            with ThreadPool(processes=MAXIMUM_NUMBER_OF_WORKERS) as pool:
+                data = list(
+                    tqdm(
+                        pool.imap(_worker, _input),
+                        total=self._progres['total'],
+                        initial=self._progres['position'],
+                        leave=self._progres['leave']
+                    )
+                )
+        else:
+            disable_progress_bar = False
+            if len(_input) == 1:
+                disable_progress_bar = True
+
+            data = list(
+                tqdm(
+                    map(_worker, _input),
+                    total=self._progres['total'],
+                    disable=disable_progress_bar,
+                    initial=self._progres['position'],
+                    leave = self._progres['leave']
+                )
+            )
+        self._update_progress_bar(len(_input))
 
         data = concat( data , dim='time',  )
-        data.coords['time'] = to_datetime64(time)
-        return data
+        data.coords['time'] = to_datetime64([self.time[it] for it in time_index])
+        return FrequencyDirectionSpectrum(data)
 
+    def _init_progress_bar(self, total):
+        if self._progres['total'] is None:
+            self._progres['total'] = total
+            self._progres['position'] = 0
+            self._progres['leave'] = True
+        else:
+            if total + self._progres['position'] == self._progres['total']:
+                self._progres['leave'] = True
+            else:
+                self._progres['leave'] = False
+
+    def _update_progress_bar(self,number):
+        self._progres['position'] += number
+
+        if self._progres['position'] == self._progres['total']:
+            self._progres['total'] = None
+            self._progres['position'] = 0
 
     def interpolate(self,latitude:Union[numpy.ndarray,float],
                         longitude:Union[numpy.ndarray,float],
                         time:Union[numpy.ndarray,numpy.datetime64,datetime]
-                    ) -> Dataset:
+                    ) -> FrequencyDirectionSpectrum:
         """
         Extract interpolated spectra at given latitudes and longitudes.
         Input can be either a single latitude and longitude pair, or a
@@ -686,14 +766,8 @@ class RestartFileStack:
                   "longitude":numpy.atleast_1d(longitude),
                   }
 
-        coordinates = [("time",to_datetime64(self.time)),
-                        ("latitude",self.latitude),
-                       ("longitude",self.longitude),
-                       ]
-
         periodic_coordinates = {"longitude":360}
 
-
         def _get_data(  indices , idims):
             time_index = indices[0]
             index = self._grid.index(latitude_index=indices[1],
@@ -706,12 +780,15 @@ class RestartFileStack:
             mask = index >= 0
 
             output[  mask,:,:] = numpy.squeeze(
-                self.__getitem__((time_index[mask],index[mask]))['variance_density'])
+                self.__getitem__(
+                    (time_index[mask],index[mask]))['variance_density']
+            )
             output[ ~mask,:,:] =numpy.nan
             return output
 
         data_shape = [ len(self.time), len(self.latitude), len(self.longitude),
                        self.number_of_frequencies,self.number_of_directions]
+
         data_coordinates = (
             ('time',to_datetime64(self.time)),
             ('latitude', self.latitude),
@@ -730,8 +807,11 @@ class RestartFileStack:
             data_period=None,
             data_discont=None
         )
+
+        self._init_progress_bar(len(latitude)*8)
         dataset = interpolator.interpolate(points)
-        return Dataset(
+
+        return FrequencyDirectionSpectrum(Dataset(
             data_vars={
                 "variance_density": (
                     ('time','frequency','direction'),dataset ),
@@ -744,12 +824,12 @@ class RestartFileStack:
                 'time':time,
                 'frequency':self.frequency,
                 'direction':self.direction
-            })
+            }))
 
     def interpolate_tracks(self,tracks:TrackSet
-                          ) -> Mapping[str,DataArray]:
-        out = {}
-        for _id,track in tracks.tracks.items():
-            out[_id] = self.interpolate(track.latitude,
-                                          track.longitude,track.time)
-        return out
+                          ) -> Dict[str, FrequencyDirectionSpectrum]:
+        return {
+            _id:self.interpolate(x.latitude,x.longitude,x.time)
+                                             for _id,x in tracks.tracks.items()
+        }
+
